Package: treeio
Title: Base Classes and Functions for Phylogenetic Tree Input and
        Output
Version: 1.37.0
Authors@R: c(
    person("Guangchuang", "Yu",     email = "guangchuangyu@gmail.com", role = c("aut", "cre"), comment = c(ORCID = "0000-0002-6485-8781")),
    person("Tommy Tsan-Yuk", "Lam", email = "tylam.tommy@gmail.com",   role = c("ctb", "ths")),
    person("Shuangbin", "Xu",       email = "xshuangbin@163.com",      role = "ctb",           comment = c(ORCID="0000-0003-3513-5362")),
    person("Bradley", "Jones",      email = "brj1@sfu.ca",             role = "ctb"),
    person("Casey", "Dunn",         email = "casey_dunn@brown.edu",    role = "ctb"),
    person("Tyler", "Bradley",      email = "tcb85@drexel.edu",        role = "ctb"),
    person("Konstantinos", "Geles", email = "konstantinos.geles@studenti.unicz.it", role = "ctb")
	  )
Description: 'treeio' is an R package to make it easier to import and
        store phylogenetic tree with associated data; and to link
        external data from different sources to phylogeny. It also
        supports exporting phylogenetic tree with heterogeneous
        associated data to a single tree file and can be served as a
        platform for merging tree with associated data and converting
        file formats.
Depends: R (>= 4.1.0)
Imports: ape, dplyr, jsonlite, magrittr, methods, rlang, stats, tibble,
        tidytree (>= 0.4.5), utils, yulab.utils (>= 0.1.6)
Suggests: Biostrings, cli, ggplot2, ggtree, igraph, knitr, rmarkdown,
        phangorn, prettydoc, purrr, testthat, tidyr, vroom, xml2, yaml
VignetteBuilder: knitr
ByteCompile: true
License: Artistic-2.0
Encoding: UTF-8
URL: https://yulab-smu.top/contribution-tree-data/
BugReports: https://github.com/YuLab-SMU/treeio/issues
biocViews: Software, Annotation, Clustering, DataImport,
        DataRepresentation, Alignment, MultipleSequenceAlignment,
        Phylogenetics
RoxygenNote: 7.3.2
Config/pak/sysreqs: cmake make libicu-dev libuv1-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-04-28 12:44:53 UTC
RemoteUrl: https://github.com/bioc/treeio
RemoteRef: HEAD
RemoteSha: 4dcc69b9062a317c9eb8313df529836bd1830096
NeedsCompilation: no
Packaged: 2026-05-30 09:41:06 UTC; root
Author: Guangchuang Yu [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-6485-8781>),
  Tommy Tsan-Yuk Lam [ctb, ths],
  Shuangbin Xu [ctb] (ORCID: <https://orcid.org/0000-0003-3513-5362>),
  Bradley Jones [ctb],
  Casey Dunn [ctb],
  Tyler Bradley [ctb],
  Konstantinos Geles [ctb]
Maintainer: Guangchuang Yu <guangchuangyu@gmail.com>
Built: R 4.6.0; ; 2026-05-30 13:58:42 UTC; unix
