# Generated by roxygen2: do not edit by hand

S3method(as.phylo,data.frame)
S3method(as.phylo,dendro)
S3method(as.phylo,ggtree)
S3method(as.phylo,igraph)
S3method(as.phylo,list)
S3method(as.phylo,matrix)
S3method(as.phylo,phylo4)
S3method(as.phylo,pvclust)
S3method(as.phylo,tbl_df)
S3method(as.treedata,data.frame)
S3method(as.treedata,ggtree)
S3method(as.treedata,matrix)
S3method(as.treedata,phylo)
S3method(as.treedata,phylo4)
S3method(as.treedata,phylo4d)
S3method(as.treedata,pml)
S3method(as.treedata,pvclust)
S3method(as.treedata,tbl_df)
S3method(find.hclust,igraph)
S3method(get.placements,jplace)
S3method(print,treedataList)
S3method(spt,igraph)
export("%<>%")
export("%>%")
export(.data)
export(MRCA)
export(Nnode)
export(Nnode2)
export(Ntip)
export(ancestor)
export(as.phylo)
export(as.treedata)
export(as_tibble)
export(child)
export(drop.tip)
export(find.hclust)
export(full_join)
export(get.data)
export(get.fields)
export(get.placements)
export(get.tree)
export(get.treetext)
export(getNodeNum)
export(inner_join)
export(is.ggtree)
export(is.rooted)
export(isTip)
export(label_branch_paml)
export(mask)
export(merge_tree)
export(nodeid)
export(nodelab)
export(offspring)
export(parent)
export(raxml2nwk)
export(read.astral)
export(read.beast)
export(read.beast.newick)
export(read.codeml)
export(read.codeml_mlc)
export(read.fasta)
export(read.hyphy)
export(read.hyphy.seq)
export(read.iqtree)
export(read.jplace)
export(read.jtree)
export(read.mcmctree)
export(read.mega)
export(read.mega_tabular)
export(read.mrbayes)
export(read.newick)
export(read.nextstrain.json)
export(read.nexus)
export(read.nhx)
export(read.paml_rst)
export(read.phylip)
export(read.phylip.seq)
export(read.phylip.tree)
export(read.phyloxml)
export(read.r8s)
export(read.raxml)
export(read.tree)
export(read.treeqza)
export(read.treetime)
export(rename_taxa)
export(rescale_tree)
export(root)
export(rootnode)
export(rtree)
export(spt)
export(tibble)
export(treedata)
export(write.beast)
export(write.beast.newick)
export(write.jplace)
export(write.jtree)
export(write.nexus)
export(write.tree)
exportClasses(jplace)
import(tidytree)
importClassesFrom(tidytree,treedata)
importFrom(ape,.compressTipLabel)
importFrom(ape,.uncompressTipLabel)
importFrom(ape,Nnode)
importFrom(ape,Ntip)
importFrom(ape,as.AAbin.character)
importFrom(ape,as.DNAbin)
importFrom(ape,as.DNAbin.character)
importFrom(ape,as.phylo)
importFrom(ape,branching.times)
importFrom(ape,checkLabel)
importFrom(ape,is.rooted)
importFrom(ape,is.ultrametric)
importFrom(ape,read.FASTA)
importFrom(ape,read.nexus)
importFrom(ape,read.tree)
importFrom(ape,reorder.phylo)
importFrom(ape,root)
importFrom(ape,rtree)
importFrom(ape,trans)
importFrom(ape,write.nexus)
importFrom(ape,write.tree)
importFrom(dplyr,filter)
importFrom(dplyr,full_join)
importFrom(dplyr,group_by)
importFrom(dplyr,inner_join)
importFrom(dplyr,mutate)
importFrom(dplyr,mutate_if)
importFrom(dplyr,n)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(dplyr,summarize)
importFrom(jsonlite,fromJSON)
importFrom(jsonlite,toJSON)
importFrom(magrittr,"%<>%")
importFrom(magrittr,"%>%")
importFrom(methods,"slot<-")
importFrom(methods,.hasSlot)
importFrom(methods,is)
importFrom(methods,missingArg)
importFrom(methods,new)
importFrom(methods,setGeneric)
importFrom(methods,slot)
importFrom(parallel,mclapply)
importFrom(rlang,.data)
importFrom(rlang,check_installed)
importFrom(rlang,enexpr)
importFrom(rlang,quo_name)
importFrom(stats,as.dist)
importFrom(stats,hclust)
importFrom(stats,setNames)
importFrom(tibble,as_tibble)
importFrom(tibble,tibble)
importFrom(tidytree,MRCA)
importFrom(tidytree,ancestor)
importFrom(tidytree,as.phylo)
importFrom(tidytree,as.treedata)
importFrom(tidytree,child)
importFrom(tidytree,drop.tip)
importFrom(tidytree,get.data)
importFrom(tidytree,get.fields)
importFrom(tidytree,get_tree_data)
importFrom(tidytree,isTip)
importFrom(tidytree,nodeid)
importFrom(tidytree,nodelab)
importFrom(tidytree,offspring)
importFrom(tidytree,parent)
importFrom(tidytree,rootnode)
importFrom(tidytree,treedata)
importFrom(yulab.utils,use_perl)
importFrom(yulab.utils,yulab_msg)
