##' @importFrom ape read.tree
##' @export
ape::read.tree

##' @importFrom ape read.nexus
##' @export
ape::read.nexus


##' @importFrom ape rtree
##' @export
ape::rtree

##' @importFrom ape write.tree
##' @export
ape::write.tree

##' @importFrom ape write.nexus
##' @export
ape::write.nexus

##' @importFrom ape Nnode
##' @export
ape::Nnode

##' @importFrom ape Ntip
##' @export
ape::Ntip

##' @importFrom ape is.rooted
##' @export
ape::is.rooted

##' @importFrom ape root
##' @export
ape::root

