##' @importFrom magrittr %>%
##' @export
magrittr::`%>%`

##' @importFrom magrittr %<>%
##' @export
magrittr::`%<>%`

##' @import tidytree
NULL

##' @importFrom tidytree get.fields
##' @export
tidytree::get.fields

##' @importFrom tidytree get.data
##' @export
tidytree::get.data
 
##' @importFrom tidytree as.treedata
##' @export
tidytree::as.treedata
 
##' @importFrom tidytree ancestor
##' @export
tidytree::ancestor

##' @importFrom tidytree parent
##' @export
tidytree::parent

##' @importFrom tidytree child
##' @export
tidytree::child

##' @importFrom tidytree offspring
##' @export
tidytree::offspring

##' @importFrom tidytree rootnode
##' @export
tidytree::rootnode

##' @importFrom tidytree nodeid
##' @export
tidytree::nodeid

##' @importFrom tidytree nodelab
##' @export
tidytree::nodelab

##' @importFrom tidytree MRCA
##' @export
tidytree::MRCA


##' @importFrom dplyr full_join
##' @export
dplyr::full_join


##' @importFrom dplyr inner_join
##' @export
dplyr::inner_join

##' @importFrom tibble as_tibble
##' @export
tibble::as_tibble

##' @importFrom tibble tibble
##' @export
tibble::tibble

##' @importFrom tidytree as.phylo
##' @export
tidytree::as.phylo

##' @importFrom rlang .data
##' @export
rlang::.data

##' @importFrom tidytree drop.tip
##' @export
tidytree::drop.tip

##' @importFrom tidytree isTip
##' @export
tidytree::isTip
