% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tree-utilities.R
\name{getNodeNum}
\alias{getNodeNum}
\alias{Nnode2}
\title{getNodeNum}
\usage{
getNodeNum(tree)

Nnode2(tree)
}
\arguments{
\item{tree}{tree object}
}
\value{
number
}
\description{
calculate total number of nodes
}
\examples{
getNodeNum(rtree(30))
Nnode2(rtree(30))
}
\author{
Guangchuang Yu
}
