% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/r8s.R
\name{read.r8s}
\alias{read.r8s}
\title{read.r8s}
\usage{
read.r8s(file)
}
\arguments{
\item{file}{r8s output log file}
}
\value{
multiPhylo object
}
\description{
parse output from r8s
}
\examples{
read.r8s(system.file("extdata/r8s", "H3_r8s_output.log", package="treeio"))
}
\author{
Guangchuang Yu
}
